from collections.abc import Iterable, Iterator, Mapping
from typing import Any, Literal

from django.core import validators  # due to weird mypy.stubtest error
from django.db.models import NOT_PROVIDED, Field
from django.db.models.base import Model
from django.db.models.fields.reverse_related import ForeignObjectRel
from django.db.models.options import _AnyField
from django.utils.choices import _ChoicesInput
from django.utils.functional import _StrOrPromise, cached_property
from typing_extensions import override

class AttributeSetter:
    def __init__(self, name: str, value: Any) -> None: ...

class CompositeAttribute:
    field: CompositePrimaryKey
    def __init__(self, field: CompositePrimaryKey) -> None: ...
    @property
    def attnames(self) -> list[str]: ...
    def __get__(self, instance: Model, cls: type[Model] | None = None) -> tuple[Any, ...]: ...
    def __set__(self, instance: Model, values: list[Any] | tuple[Any] | None) -> None: ...

class CompositePrimaryKey(Field[Any, Any]):
    field_names: tuple[str]
    descriptor_class: type[CompositeAttribute]
    def __init__(
        self,
        *args: str,
        verbose_name: _StrOrPromise | None = None,
        name: str | None = None,
        primary_key: Literal[True] = True,
        max_length: int | None = None,
        unique: bool = False,
        blank: Literal[True] = True,
        null: bool = False,
        db_index: bool = False,
        rel: ForeignObjectRel | None = None,
        default: type[NOT_PROVIDED] = ...,
        editable: Literal[False] = False,
        serialize: bool = True,
        unique_for_date: str | None = None,
        unique_for_month: str | None = None,
        unique_for_year: str | None = None,
        choices: _ChoicesInput | None = None,
        help_text: _StrOrPromise = "",
        db_column: None = None,
        db_tablespace: str | None = None,
        auto_created: bool = False,
        validators: Iterable[validators._ValidatorCallable] = (),
        error_messages: Mapping[str, _StrOrPromise] | None = None,
        db_comment: str | None = None,
        db_default: type[NOT_PROVIDED] = ...,
    ) -> None: ...
    @cached_property
    def fields(
        self,
    ) -> tuple[_AnyField, ...]: ...
    @cached_property
    def columns(self) -> tuple[str, ...]: ...
    def __iter__(self) -> Iterator[_AnyField]: ...
    def __len__(self) -> int: ...
    @override
    def get_pk_value_on_save(self, instance: Model) -> tuple: ...  # actual type is tuple of field.value_from_object

def unnest(fields: Iterable[Field[Any, Any]]) -> list[Field[Any, Any]]: ...



from typing import Any

from django.db.backends.base.features import BaseDatabaseFeatures
from django.db.backends.oracle.base import DatabaseWrapper
from django.utils.functional import cached_property
from typing_extensions import override

class DatabaseFeatures(BaseDatabaseFeatures):
    connection: DatabaseWrapper
    interprets_empty_strings_as_nulls: bool
    has_select_for_update: bool
    has_select_for_update_nowait: bool
    has_select_for_update_skip_locked: bool
    has_select_for_update_of: bool
    select_for_update_of_column: bool
    can_return_columns_from_insert: bool
    supports_subqueries_in_group_by: bool
    supports_transactions: bool
    supports_timezones: bool
    has_native_duration_field: bool
    can_defer_constraint_checks: bool
    supports_partially_nullable_unique_constraints: bool
    supports_deferrable_unique_constraints: bool
    truncates_names: bool
    supports_tablespaces: bool
    supports_sequence_reset: bool
    can_introspect_materialized_views: bool
    atomic_transactions: bool
    nulls_order_largest: bool
    requires_literal_defaults: bool
    closed_cursor_error_class: Any
    supports_select_for_update_with_limit: bool
    supports_temporal_subtraction: bool
    ignores_table_name_case: bool
    supports_index_on_text_field: bool
    create_test_procedure_without_params_sql: str
    create_test_procedure_with_int_param_sql: str
    supports_callproc_kwargs: bool
    supports_over_clause: bool
    supports_frame_range_fixed_distance: bool
    supports_ignore_conflicts: bool
    max_query_params: int
    supports_partial_indexes: bool
    supports_slicing_ordering_in_compound: bool
    allows_multiple_constraints_on_same_fields: bool
    supports_json_field_contains: bool
    can_return_rows_from_update: bool
    supports_aggregate_order_by_clause: bool
    supports_any_value: bool
    supports_tuple_comparison_against_subquery: bool
    supports_json_negative_indexing: bool

    @cached_property
    @override
    def django_test_skips(self) -> dict[str, set[str]]: ...  # type: ignore[override]
    @cached_property
    @override
    def introspected_field_types(self) -> dict[str, str]: ...  # type: ignore[override]
    @cached_property
    @override
    def test_collations(self) -> dict[str, str | None]: ...  # type: ignore[override]
    @cached_property
    @override
    def supports_collation_on_charfield(self) -> bool: ...  # type: ignore[override]
    @cached_property
    @override
    def supports_primitives_in_json_field(self) -> bool: ...  # type: ignore[override]
    @cached_property
    @override
    def supports_frame_exclusion(self) -> bool: ...  # type: ignore[override]
    @cached_property
    @override
    def supports_comparing_boolean_expr(self) -> bool: ...  # type: ignore[override]
    @cached_property
    def supports_aggregation_over_interval_types(self) -> bool: ...
    @cached_property
    @override
    def bare_select_suffix(self) -> str: ...  # type: ignore[override]
    @cached_property
    @override
    def supports_tuple_lookups(self) -> bool: ...  # type: ignore[override]



from collections.abc import AsyncIterable, AsyncIterator, Iterable, Iterator
from html.parser import HTMLParser
from io import BytesIO
from re import Pattern
from typing import Any, ClassVar, TypeVar, overload

from django.db.models.base import Model
from django.utils.functional import SimpleLazyObject, _StrOrPromise, _StrPromise, cached_property
from typing_extensions import override

_StrOrPromiseT = TypeVar("_StrOrPromiseT", bound=_StrOrPromise)
_StrOrPromiseOrNoneT = TypeVar("_StrOrPromiseOrNoneT", bound=_StrOrPromise | None)

def capfirst(x: _StrOrPromiseOrNoneT) -> _StrOrPromiseOrNoneT: ...

re_newlines: Pattern[str]
re_camel_case: Pattern[str]

def wrap(text: _StrOrPromiseT, width: int) -> _StrOrPromiseT: ...
def add_truncation_text(text: str, truncate: str | None = None) -> str: ...
def calculate_truncate_chars_length(length: int, replacement: str | None) -> int: ...

class TruncateHTMLParser(HTMLParser):
    TruncationCompleted: ClassVar[type[Exception]]
    def __init__(self, *, length: int, replacement: str | None, convert_charrefs: bool = True) -> None: ...
    @cached_property
    def void_elements(self) -> frozenset[str]: ...

class TruncateCharsHTMLParser(TruncateHTMLParser):
    def process(self, data: str) -> tuple[str, str]: ...

class TruncateWordsHTMLParser(TruncateHTMLParser):
    def process(self, data: str) -> tuple[str, str]: ...

class Truncator(SimpleLazyObject):
    def __init__(self, text: Model | str) -> None: ...
    def chars(self, num: int, truncate: str | None = None, html: bool = False) -> str: ...
    def words(self, num: int, truncate: str | None = None, html: bool = False) -> str: ...

def get_valid_filename(name: _StrOrPromiseT) -> _StrOrPromiseT: ...
@overload
def get_text_list(list_: list[str], last_word: str = ...) -> str: ...
@overload
def get_text_list(list_: list[_StrOrPromise], last_word: _StrOrPromise = ...) -> _StrOrPromise: ...
def normalize_newlines(text: _StrOrPromiseT) -> _StrOrPromiseT: ...
def phone2numeric(phone: _StrOrPromiseT) -> _StrOrPromiseT: ...
def compress_string(s: bytes, *, max_random_bytes: int | None = None) -> bytes: ...

class StreamingBuffer(BytesIO):
    @override
    def read(self) -> bytes: ...  # type: ignore[override]

def compress_sequence(sequence: Iterable[bytes], *, max_random_bytes: int | None = None) -> Iterator[bytes]: ...
def acompress_sequence(
    sequence: AsyncIterable[bytes], *, max_random_bytes: int | None = None
) -> AsyncIterator[bytes]: ...

smart_split_re: Pattern[str]

def smart_split(text: str) -> Iterator[str]: ...
def unescape_string_literal(s: _StrOrPromiseT) -> _StrOrPromiseT: ...
def slugify(value: _StrOrPromiseT, allow_unicode: bool = False) -> _StrOrPromiseT: ...
def camel_case_to_spaces(value: str) -> str: ...
def format_lazy(format_string: _StrOrPromise, *args: Any, **kwargs: Any) -> _StrPromise: ...
